﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Collections;

namespace VisualLinq
{
    public class VisualGroupBy<TSource, TResult, TKey> : VisualChild<IGrouping<TKey,TResult>>
    {
        readonly VisualEnumerable<TSource> source;
        readonly Func<TSource, TResult> elementSelector;
        readonly Func<TSource, TKey> keySelector;

        public VisualGroupBy(VisualEnumerable<TSource> source, Expression<Func<TSource,TKey>> keySelector, Expression<Func<TSource, TResult>> elementSelector, bool showElementSelector)
            : base(source, FormatGroupBy(showElementSelector, elementSelector, keySelector))
        {
            this.source = source;
            this.elementSelector = elementSelector.Compile();
            this.keySelector = keySelector.Compile();
        }

        private static string FormatGroupBy(bool showElementSelector, Expression<Func<TSource, TResult>> elementSelector, Expression<Func<TSource, TKey>> keySelector)
        {
            return "group " + (showElementSelector ? FormatExpression(elementSelector.Body) : keySelector.Parameters[0].Name) +
                   " by " + FormatExpression(keySelector.Body);
        }

        public override IEnumerator<IGrouping<TKey,TResult>> GetEnumerator()
        {
            List<TSource> input = new List<TSource>();
            foreach (TSource element in source)
            {
                AnimateDataMovement(element.ToString());
                input.Add(element);
            }

            foreach (IGrouping<TKey,TResult> result in input.GroupBy(keySelector, elementSelector))            
            {
                yield return new Grouping(result);
            }
        }

        internal override void ThreadedDispose()
        {
            base.ThreadedDispose();
            source.ThreadedDispose();
        }

        class Grouping : IGrouping<TKey,TResult>
        {
            IGrouping<TKey,TResult> original;

            internal Grouping(IGrouping<TKey, TResult> original)
            {
                this.original = original;
            }

            public TKey Key 
            { 
                get
                {
                    return original.Key;
                }
            }

            public IEnumerator<TResult> GetEnumerator()
            {
                return original.GetEnumerator();
            }

            IEnumerator IEnumerable.GetEnumerator()
            {
                return GetEnumerator();
            }

            public override string ToString()
            {
                return string.Format("Key: {0}, Data: {1}", Key, string.Join(", ", original.Select(x => x.ToString()).ToArray()));
            }
        }
    }
}
