﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Linq.Expressions;
using System.Windows.Controls;
using System.Windows.Media.Animation;
using System.Text.RegularExpressions;

namespace VisualLinq
{
    public class VisualSelect<TSource,TResult> : VisualChild<TResult>
    {
        readonly VisualEnumerable<TSource> source;
        readonly Func<TSource, TResult> projection;

        public VisualSelect(VisualEnumerable<TSource> source, Expression<Func<TSource, TResult>> projection)
            : base(source, "select "+FormatExpression(projection.Body))
        {
            this.source = source;
            this.projection = projection.Compile();
        }

        public override IEnumerator<TResult> GetEnumerator()
        {
            foreach (TSource element in source)
            {
                TResult result = projection(element);
                AnimateDataMovement(element.ToString());
                AnimateTransition(element.ToString(), result.ToString());
                yield return result;
            }
        }

        void AnimateTransition(string from, string to)
        {
            Animate(() =>
            {
                Storyboard storyboard = new Storyboard();
                Duration duration = new Duration(TimeSpan.FromSeconds(1));
                DoubleAnimation fadeIn = new DoubleAnimation(0, 1, duration);
                DoubleAnimation fadeOut = new DoubleAnimation(1, 0, duration);
                storyboard.Children.Add(fadeIn);
                storyboard.Children.Add(fadeOut);
                TextBlock fromText = new TextBlock { Text = from, Name = "fromText" };
                Canvas.RegisterName(fromText.Name, fromText);
                Canvas.Children.Add(fromText);

                TextBlock toText = new TextBlock { Text = to, Name = "toText" };
                Canvas.RegisterName(toText.Name, toText);
                Canvas.Children.Add(toText);

                Canvas.SetLeft(fromText, Canvas.GetLeft(Box) + Box.Width + 10);
                Canvas.SetTop(fromText, Canvas.GetTop(Box));
                Canvas.SetLeft(toText, Canvas.GetLeft(fromText));
                Canvas.SetTop(toText, Canvas.GetTop(fromText));
                Storyboard.SetTargetName(fadeIn, toText.Name);
                Storyboard.SetTargetName(fadeOut, fromText.Name);
                Storyboard.SetTargetProperty(fadeIn, new PropertyPath(UIElement.OpacityProperty));
                Storyboard.SetTargetProperty(fadeOut, new PropertyPath(UIElement.OpacityProperty));

                storyboard.Completed += delegate
                {
                    Canvas.Children.Remove(toText);
                    Canvas.Children.Remove(fromText);
                    Canvas.UnregisterName(toText.Name);
                    Canvas.UnregisterName(fromText.Name);
                };

                return storyboard;
            });
        }

        internal override void ThreadedDispose()
        {
            base.ThreadedDispose();
            source.ThreadedDispose();
        }
    }
}
