﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Windows.Threading;
using System.Windows.Media.Animation;
using System.Windows;
using System.Windows.Controls;

namespace VisualLinq
{
    public class VisualWhere<T> : VisualChild<T>
    {
        readonly VisualEnumerable<T> source;
        readonly Func<T, bool> predicate;

        public VisualWhere(VisualEnumerable<T> source,
                           Expression<Func<T, bool>> predicate)
            : base(source, "where "+predicate.Body.ToString())
        {
            this.source = source;
            this.predicate = predicate.Compile();
        }

        public override IEnumerator<T> GetEnumerator()
        {
            foreach (T element in source)
            {
                AnimateDataMovement(element.ToString());
                AnimatePredicate(element.ToString(), predicate(element));
                if (predicate(element))
                {
                    yield return element;
                }
            }
        }

        void AnimatePredicate(string text, bool matches)
        {
            Animate(() =>
            {
                Storyboard storyboard = new Storyboard();
                Duration duration = new Duration(TimeSpan.FromSeconds(1));
                DoubleAnimation fadeIn = new DoubleAnimation(0, 1, duration);
                DoubleAnimation fadeOut = new DoubleAnimation(1, 1, duration);
                storyboard.Children.Add(fadeIn);
                storyboard.Children.Add(fadeOut);
                TextBlock fromText = new TextBlock { Text = text+"? ", Name = "fromText" };
                Canvas.RegisterName(fromText.Name, fromText);
                Canvas.Children.Add(fromText);

                TextBlock toText = new TextBlock { Text = matches.ToString(), Name = "toText" };
                Canvas.RegisterName(toText.Name, toText);
                Canvas.Children.Add(toText);

                fromText.Measure(new Size(10000, 10000));

                Canvas.SetLeft(fromText, Canvas.GetLeft(Box) + Box.Width + 10);
                Canvas.SetTop(fromText, Canvas.GetTop(Box));
                Canvas.SetLeft(toText, Canvas.GetLeft(fromText) + fromText.DesiredSize.Width);
                Canvas.SetTop(toText, Canvas.GetTop(fromText));
                Storyboard.SetTargetName(fadeIn, toText.Name);
                Storyboard.SetTargetName(fadeOut, fromText.Name);
                Storyboard.SetTargetProperty(fadeIn, new PropertyPath(UIElement.OpacityProperty));
                Storyboard.SetTargetProperty(fadeOut, new PropertyPath(UIElement.OpacityProperty));

                storyboard.Completed += delegate
                {
                    Canvas.Children.Remove(toText);
                    Canvas.Children.Remove(fromText);
                    Canvas.UnregisterName(toText.Name);
                    Canvas.UnregisterName(fromText.Name);
                };

                return storyboard;
            });
        }

        internal override void ThreadedDispose()
        {
            base.ThreadedDispose();
            source.ThreadedDispose();
        }
    }
}
